// Vincia.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2014 P.Z. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file contains global header information for Vincia.

#ifndef VINCIA_H
#define VINCIA_H

// Include Pythia 8 headers and name space
#include "Pythia8/Pythia.h"

// Now include headers that need this information
#include "VinciaAntenna.h"
#include "VinciaCommon.h"
#include "VinciaColourReconnections.h"
#include "VinciaFunctions.h"
#include "VinciaShower.h"
#include "VinciaMatching.h"
#include "VinciaMGInterface.h"

using namespace Pythia8;
using namespace Vincia;

// Define namespace inside which Vincia lives
namespace Vincia { 
  
  //*********
  
  // The VinciaPlugin class
  
  class VinciaPlugin {
    
  public:
    
    VinciaPlugin() {
      cerr << "\n (VinciaPlugin():) cannot construct VinciaPlugin :(\n\n"
      <<"  Sorry, the Vincia plugin must be constructed with a pointer"
      <<" to an\n  explicit Pythia object (since it's a plugin)." 
      <<" Please use one of \n\n"
      <<"     VinciaPlugin vincia(Pythia* pythia);\n"
      <<"     VinciaPlugin vincia(Pythia* pythia, string commandFile);\n"
      <<"     VinciaPlugin vincia(Pythia* pythia, string commandFile, string xmlDir);\n"
      <<"     VinciaPlugin vincia(Pythia* pythia, string commandFile, string xmlDir, string xmlIndexFile);\n"
      <<"\n";
    }
    
    // Constructor
    VinciaPlugin(Pythia* pythia, string commandFile="none", 
                 string xmlDir="xmldoc", 
                 string xmlIndexFile="VinciaIndex.xml") {      
      
      // First set pointers to Pythia classes 
      couplingsPtr      = &pythia->couplings;
      infoPtr           = &pythia->info;
      particleDataPtr   = &pythia->particleData;
      partonSystemsPtr  = &pythia->partonSystems;
      rndmPtr           = &pythia->rndm;
      settingsPtr       = &pythia->settings;
      // Initial value for pointer to user hooks.
      userHooksPtr      = 0;
      
      // Find path to data files, i.e. xmldoc directory location.
      // Environment variable takes precedence, else use constructor input.
      string xmlPath        = "";
      bool existsVINCIADATA = false;
      const char* VINCIADATA = "VINCIADATA";
      char* envPath = getenv(VINCIADATA);
      if (envPath != 0 && *envPath != '\0') {
        int i = 0;
        while (*(envPath+i) != '\0') xmlPath += *(envPath+(i++));
        existsVINCIADATA = true;
      }
      else xmlPath = xmlDir;
      if (xmlPath[ xmlPath.length() - 1 ] != '/') xmlPath += "/";      
      // Add the VINCIA list of parameters to the PYTHIA 8 settings database
      settingsPtr->init(xmlPath+xmlIndexFile, true); 
      addAntennaSettings();    
      
      // Init default shower pointers and default settings
      shower.functionsPtr           = &functions;
      shower.matchingPtr            = &matching;
      shower.mgInterfacePtr         = &mgInterface;
      shower.resolutionPtr          = &resolution;
      shower.subtractionTermsPtr    = &subtractionTerms;
      shower.vinCluPtr              = &vinClu;
      shower.vinComPtr              = &vinCom;
      shower.colourReconnectionsPtr = &colourReconnections;
      shower.initPtr(infoPtr, settingsPtr, particleDataPtr, rndmPtr,
                     couplingsPtr, partonSystemsPtr, userHooksPtr) ;
      
      // Initialize default antenna set
      antennaSetDef.initPtr(&functions, 
                            particleDataPtr, rndmPtr, settingsPtr);
      antennaSetDef.init(false,0);
      antennaSetSctDef.initPtr(&functions, 
                               particleDataPtr, rndmPtr, settingsPtr); 
      antennaSetSctDef.init(true,0);

      // Read command file 1st time
      if (commandFile != "none") pythia->readFile(commandFile); 
      verbose = 0;
      int verboseGlo = settingsPtr->mode("Vincia:verbose");
      
      if (verboseGlo >= 6)
        cout<<" VinciaPlugin() begin ------------------"<<endl;      
      
      // Vincia on/off
      vinciaOn = settingsPtr->flag("Vincia");
      
      // Initialize standard Antennae       
      if (vinciaOn) {
        
        if (verboseGlo >= 2) {
          if (existsVINCIADATA) 
            cout<<" VinciaPlugin(): VINCIADATA environment variable ok."<<endl;
          else cout<<" VinciaPlugin(): VINCIADATA environment variable undefined. Using constructor instead."<<endl; 
        }
        if (verboseGlo >= 1) {
          cout<<" VinciaPlugin(): using configuration directory : "
          <<xmlPath<<endl;
        }
        
        if (verboseGlo >= 2)
          cout<<" VinciaPlugin(): initializing antennae..."<<endl;
        
        // Check if using uncertainty bands          
        if (verboseGlo >= 3)
          cout<<" VinciaPlugin(): initializing MAX antennaSet"<<endl;
        antennaSetMax.initPtr(&functions, 
                              particleDataPtr, rndmPtr, settingsPtr);
        antennaSetMax.init(false,1);
        antennaSetSctMax.initPtr(&functions, 
                                 particleDataPtr, rndmPtr, settingsPtr); 
        antennaSetSctMax.init(true,1);
        
        if (verboseGlo >= 3) 
          cout<<" VinciaPlugin(): initializing MIN antennaSet"<<endl;
        antennaSetMin.initPtr(&functions,
                              particleDataPtr, rndmPtr, settingsPtr);
        antennaSetMin.init(false,-1);
        antennaSetSctMin.initPtr(&functions,
                                 particleDataPtr, rndmPtr, settingsPtr); 
        antennaSetSctMin.init(true,-1);
        
        // If filename given for non-default antenna presets, read file.
        string antennaFile=settingsPtr->word("Vincia:antennaFile");
        if (antennaFile != "none" && antennaFile != "") { 
          if (verboseGlo >= 3) 
            cout << " VinciaPlugin(): reading USR antenna presets from "
            <<antennaFile<<endl;
          pythia->readFile(antennaFile);
        }
        
        // If filename given for tune presets, read file.
	// Get rid of this, replace with tune class instead that 
	// sets parameters and allows user mods on top.
        string tuneFile=settingsPtr->word("Vincia:tuneFile");
        if (tuneFile != "none" && tuneFile != "") { 
          if (verboseGlo >= 3) 
            cout << " VinciaPlugin(): reading TUNE presets from "
            <<tuneFile<<endl;
          pythia->readFile(tuneFile);
        }
        
        // Read command file 2nd time (possibility to overwrite tune settings)
        if (commandFile != "none") pythia->readFile(commandFile); 
        
        // Set pointers for global and sector antenna sets
	// Initialization done when VinciaShower initializes
        antennaSet.initPtr(&functions, 
                           particleDataPtr, rndmPtr, settingsPtr);
	antennaSetSct.initPtr(&functions, 
                              particleDataPtr, rndmPtr, settingsPtr); 
        
        // Now set verbose
        verbose = verboseGlo;
        
        if (verboseGlo >= 2)
          cout<<" VinciaPlugin(): initializing pointers..."<<endl;
        
        // Hand antenna set pointers to shower and matching objects
        shower.initAntPtr(&antennaSet,&antennaSetDef,
			  &antennaSetMax,&antennaSetMin);
        matching.initAntPtr(&antennaSet,&antennaSetDef,
			    &antennaSetMax,&antennaSetMin);
	shower.initAntSctPtr(&antennaSetSct,&antennaSetSctDef,
			     &antennaSetSctMax,&antennaSetSctMin);
	matching.initAntSctPtr(&antennaSetSct,&antennaSetSctDef,
			       &antennaSetSctMax,&antennaSetSctMin);
        
        // Set Pythia 8 shower pointers to Vincia 
        // (user must manually change back if wanting partial Pythia, 
        //  see vincia06.cc example)
        pythia->setShowerPtr(&shower,&shower,0);    
        
      } 
      
      // If VINCIA off, let us know about it
      else {
        if (verbose >= 2) printOut("VinciaPlugin","Vincia is switched off");
      }
      
      // Pass pointers on to objects that require them
      resolution.initPtr(settingsPtr);
      rambo.initPtr(rndmPtr);
      subtractionTerms.initPtr(settingsPtr);
      vinCom.initPtr(particleDataPtr,settingsPtr);
      vinClu.initPtr(rndmPtr,settingsPtr,&vinCom);
      mgInterface.initPtr(couplingsPtr,particleDataPtr,settingsPtr,&vinCom);
      matching.initPtr(&mgInterface, particleDataPtr, &resolution,
                       rndmPtr, settingsPtr, 
		       &subtractionTerms, &vinClu, &vinCom, &functions);    
      colourReconnections.initPtr(rndmPtr,settingsPtr);
      
      // Initialize a few objects
      vinClu.init();
      subtractionTerms.init();
      
      if (verboseGlo >= 6) 
        cout<<" VinciaPlugin() end ------------------"<<endl;      
    } 
    
    //*********
    
    // Add placeholders for antenna functions in Settings database
    // (only ones with non-zero coefficients will be saved later,
    // but the definitions are needed in case user wants to update.)
    
    void addAntennaSettings() {
      
      for (int iAnt=0; iAnt<=10; iAnt++) {
        string name;
        if (iAnt == iQQemit) name="Vincia:QQemit";
        else if (iAnt == iQGemit) name="Vincia:QGemit";
        else if (iAnt == iQGsplit) name="Vincia:QGsplit";
        else if (iAnt == iGGemit) name="Vincia:GGemit";
        else if (iAnt == iGGsplit) name="Vincia:GGsplit";
        else continue;
        
        // Check if global switch exists
        if ( ! settingsPtr->isFlag(name) ) 
          settingsPtr->addFlag(name,false);
        
        //Check if chargeFactor exists
        string parm(name+":chargeFactor");
        if ( ! settingsPtr->isParm(parm) )
          settingsPtr->addParm(parm,0.0,false,false,0.0,0.0); 
        
        // Add the C, D, E, K Laurent coefficients
        for (int i=-1;i<=LaurentIndexMax;i++) {
          string iStr(num2str(i,0));
          for (int j=-1;j<=LaurentIndexMax;j++) {
            string jStr(num2str(j,0));
            parm=name+":LaurentC("+iStr+","+jStr+")";
            if ( ! settingsPtr->isParm( parm ) )  
              settingsPtr->addParm(parm,0.0,false,false,0.0,0.0); 
          }	
          parm=name+":LaurentD("+iStr+")";
          if ( !  settingsPtr->isParm(parm) )  
            settingsPtr->addParm(parm,0.0,false,false,0.0,0.0);   
          parm=name+":LaurentE("+iStr+")";
          if ( !  settingsPtr->isParm(parm) )  
            settingsPtr->addParm(parm,0.0,false,false,0.0,0.0);   
        }
        // Add the K coefficient
        parm=name+":LaurentK";
        if ( !  settingsPtr->isParm(parm) )  
          settingsPtr->addParm(parm,0.0,false,false,0.0,0.0);   
        // ...
      }
    }  
    
    //*********
    
    // Automatically set verbose level in all inheriting members
    void setVerbose(int verboseIn) {
      vinCom.setVerbose(verboseIn);
      resolution.setVerbose(verboseIn);
      shower.setVerbose(verboseIn);
      matching.setVerbose(verboseIn);
      colourReconnections.setVerbose(verboseIn);
      mgInterface.setVerbose(verboseIn);
      vinClu.setVerbose(verboseIn);
    }
    
    // Main VINCIA on/off switch
    bool vinciaOn;
    
    // Instantiate Vincia Objects
    VinciaCommon        vinCom;
    Resolution          resolution;
    VinciaShower        shower;
    Matching            matching;
    ColourReconnections colourReconnections;
    SubtractionTerms    subtractionTerms;

    // Auxiliary objects
    Functions           functions;
    MGInterface         mgInterface;
    Rambo               rambo;
    VinClu              vinClu;  
    
    // Vectors of physical Antennae with uncertainty bands
    AntennaSet antennaSet;
    AntennaSet antennaSetDef;
    AntennaSet antennaSetMax;
    AntennaSet antennaSetMin;
    //
    AntennaSet antennaSetSct;
    AntennaSet antennaSetSctDef;
    AntennaSet antennaSetSctMax;
    AntennaSet antennaSetSctMin;
    
    
    // Pointers to Pythia classes
    Couplings*     couplingsPtr;
    Info*          infoPtr; 
    ParticleData*  particleDataPtr;
    PartonSystems* partonSystemsPtr;
    Rndm*          rndmPtr; 
    Settings*      settingsPtr; 
    UserHooks*     userHooksPtr;
    
    // Shortcut for post-Vincia event weight
    double weight(int iVar=0) {
      if (settingsPtr->flag("Vincia"))
        return shower.weight(iVar);
      else 
        return infoPtr->weight();
    }
    int nVariations() {return shower.nVariations();}
    bool reweightingOccurred() { return shower.reweightingOccurred;}
    
    // Utilities for printing Info and Internal histograms
    void printInfo( ostream& os = cout ) { shower.printInfo(os); }
    void printHistos( ostream& os = cout ) { shower.printHistos(os); }
    void writeHistos( string fileName = "vincia", string lastName = "dat" ) { 
      shower.writeHistos(fileName,lastName); 
    }
    const Hist& getDiagnosticHistogram( string name ) {
      return shower.getDiagnosticHistogram(name);
    }
    
    private :
    
    // Verbose level
    int verbose;
    
  }; // end Vincia plugin
  
} // end Vincia namespace
#endif
