// VinciaSubtractionterms.cc is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2013 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// Function definitions for VINCIA's SubtractionTerms class.

#include "VinciaSubtractionTerms.h"

namespace Vincia {

  // Initialization function
  bool SubtractionTerms::init() {
    // Check that pointers initialized
    if (! isInitPtr) {
      printOut("SubtractionTerms::init","Cannot initialize, pointers not set.");
      return false;
    }
    // Set verbose level
    verbose   = settingsPtr->mode("Vincia:verbose");
    isInit    = true;
    return isInit;
  }

  //=====================================================================
  // Differential subtraction terms (= sub-antennae)  

  // Quark-antiquark emission = a30 (qi, gj, qbark)
  double SubtractionTerms::AqqEmit(double yij, double yjk) {
    if (yij <= 0. || yjk <= 0.) {
      printOut("VinciaSubtractionTerms::AqqEmit",
               "Input parameter non physical");
      return 0.;
    }
    return 2.*(1.-yij-yjk)/yij/yjk + yij/yjk + yjk/yij;
  }

  // Quark-gluon emission = d30 (qi, gj, gk)
  double SubtractionTerms::AqgEmit(double yij, double yjk) {
    if (yij <= 0. || yjk <= 0.) {
      printOut("VinciaSubtractionTerms::AqqEmit",
               "Input parameter non physical");
      return 0.;
    }
    double yik = 1.-yij-yjk;
    return 2.*yik/yij/yjk + (yik*yjk + pow2(yjk))/yij + yij*yik/yjk 
      + 2.5 + 0.5*yjk;
  }

  // Quark-gluon splitting = e30 (qi, qbar'j, q'k)
  double SubtractionTerms::AqgSplit(double yij, double yjk) {
    if (yij <= 0. || yjk <= 0.) {
      printOut("VinciaSubtractionTerms::AqqEmit",
               "Input parameter non physical");
      return 0.;
    }
    double yik = 1.-yij-yjk;
    return 0.5*( (pow2(yij) + pow2(yik))/yjk + yij + yik );
  }

  //=====================================================================
  // Constructing the expanded Sudakovs from the predefined integrands

  // Quark-antiquark emission
  double SubtractionTerms::IqqEmit(double yij, double yjk, int typeEv){

    double y3,DL=0,SL1=0,SL2=0,Iqq=0; 
    // Sanity check 
    if( (yij < 0) || (yjk > 1) || ( yij + yjk > 1 ) ){
      printOut("VinciaSubtractionTerms::IqqEmit", 
               "Input parameter non physical");
      return 0;
    }     
    if( typeEv == 1 ){
      y3 = SubtractionTerms::yEpT( yij, yjk);
      DL = SubtractionTerms::DLStpT( y3 );
      SL2 = SubtractionTerms::SL12StpT( y3 );
    }
    else if( typeEv == 2 ){
      double xi;
      xi = min( yij, yjk )/( 1 - min( yij, yjk ) );
      DL = SubtractionTerms::DLStmD( xi );
      SL1 = SubtractionTerms::SL1StmD( xi );
      SL2 = SubtractionTerms::SL2StmD( xi );       
    }
    else if( typeEv == 3 ){
      y3 = SubtractionTerms::yEEn( yij, yjk );
      DL = SubtractionTerms::DLEn( y3 );
      SL1 = SubtractionTerms::SL1En( y3 );
      SL2 = SubtractionTerms::SL2En( y3 );       
    }
    else{ 
      printOut("VinciaSubtractionTerms::IqqEmit",
               "Unfamiliar type of evolution variable");
      return 0;
    }
    Iqq = CA*(DL - SL1 + SL2);

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::IqqEmit",
                                   "Iqq = "+num2str(Iqq));  
    }      
  return Iqq;
  }

  // Quark-gluon emission, sij denotes the invariant of the emitting dipole
  double SubtractionTerms::IqgEmit(double yij, double yjk, int typeEv, int typeOr){

    double DL=0,SL1=0,SL2=0,SL3=0,Iqg,Cst=0,C00,C01,C10; 

    // GGG d30 antenna contants
    C00 = 2.500;
    C01 = -0.500;
    C10 = -1.00;

    // Sanity check
    if( (yij < 0) || (yjk < 0)  ){
      printOut("VinciaSubtractionTerms::IqgEmit",
               "Input parameter non physical");
      return 0;
    }     
    if( typeOr==1 ){
      if( typeEv==1 ){
        // pT strong ordering
        if( (4*yij < 1) && ( yij != 0 ) ){  
          DL = SubtractionTerms::DLStpT( 4*yij );
          SL2 = SubtractionTerms::SL12StpT( 4*yij );
          SL3 = SubtractionTerms::SL3StpT( 4*yij );
          Cst = SubtractionTerms::CStpT( 4*yij,C00,C01,C10 );
        }
        if( (4*yjk < 1) && ( yjk != 0 ) ){  
          DL = DL + SubtractionTerms::DLStpT( 4*yjk );
          SL2 = SL2 + SubtractionTerms::SL12StpT( 4*yjk );
          SL3 = SL3 + SubtractionTerms::SL3StpT( 4*yjk );
          Cst = Cst + SubtractionTerms::CStpT( 4*yjk,C00,C01,C10 );
        }
      }
      else if( typeEv==2 ){
        // mD strong ordering
        if( yij == yjk ){ 
          printOut("VinciaSubtractionTerms::IqgEmit",
                   "Invariants are equal cannot determine minimum.");
          return 0;
	}
	else if ( ( max( yij, yjk )/min( yij, yjk ) > 2 ) ){
          double xi;
          xi = min( yij, yjk )/( max( yij, yjk ) - min( yij, yjk ) );
          DL = SubtractionTerms::DLStmD( xi );
          SL1 = SubtractionTerms::SL1StmD( xi );
          SL2 = SubtractionTerms::SL2StmD( xi );
          SL3 = SubtractionTerms::SL3StmD( xi );
          Cst = SubtractionTerms::CStmD( xi,C00,C01,C10 );
	}
      }
      else if( typeEv==3 ){
      }
      else{ 
        printOut("VinciaSubtractionTerms::IqgEmit","Unknown type of evolution");
        return 0;
      }
    }
    else if( typeOr==2 ){
      if( typeEv==1 ){
        // pT smooth ordering
        DL = SubtractionTerms::DLSmpT(yij) + SubtractionTerms::DLSmpT(yjk);
        SL1 = SubtractionTerms::SL1SmpT(yij) + SubtractionTerms::SL1SmpT(yjk);
        SL2 = SubtractionTerms::SL2SmpT(yij) + SubtractionTerms::SL2SmpT(yjk);
        SL3 = SubtractionTerms::SL3SmpT(yij) + SubtractionTerms::SL3SmpT(yjk);
        Cst = SubtractionTerms::CSmpT( yij,C00,C01,C10 ) 
	      + SubtractionTerms::CSmpT( yjk,C00,C01,C10 );
      }
      else if( typeEv==2 ){
        if( yij == yjk ){ 
          printOut("VinciaSubtractionTerms::IqgEmit",
                   "Invariants are equal, cannot determine min.");
          return 0;
	} 
        else {   
        // mD smooth ordering 
          DL = 2*SubtractionTerms::DLSmmD( min(yij,yjk)/yij ) 
            + 2*SubtractionTerms::DLSmmD( min(yij,yjk)/yjk ); 
          SL1 = 2*SubtractionTerms::SL1SmmD( min(yij,yjk)/yij ) 
            + 2*SubtractionTerms::SL1SmmD( min(yij,yjk)/yjk ); 
          SL2 = 2*SubtractionTerms::SL2SmmD( min(yij,yjk)/yij ) 
            + 2*SubtractionTerms::SL2SmmD( min(yij,yjk)/yjk ); 
          SL3 = SubtractionTerms::SL3SmmD( min(yij,yjk)/yij ) 
            + SubtractionTerms::SL3SmmD( min(yij,yjk)/yjk ); 
          Cst = SubtractionTerms::CSmmD( min(yij,yjk)/yij,C00,C01,C10 ) 
            + SubtractionTerms::CSmmD( min(yij,yjk)/yij,C00,C10,C01 ) 
            + SubtractionTerms::CSmmD( min(yij,yjk)/yjk,C00,C01,C10 ) 
            + SubtractionTerms::CSmmD( min(yij,yjk)/yjk,C00,C10,C01 ); 
	}
      }
      else if( typeEv == 3 ){ 
	// energy smooth ordering
      }
      else { printOut("VinciaSubtractionTerms::IqgEmit",
                      "Unknown type of evolution."); 
        return 0;
      }
    }
    else { printOut("VinciaSubtractionTerms::IqqEmit",
                    "Unfamiliar type of ordering");
      return 0;
    }
    Iqg = CA*(DL - SL1 + SL2 - SL3 + Cst);

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::IqgEmit",
                                   "Iqg = "+num2str(Iqg));    
    }    
    return Iqg;
  }

  // Quark-gluon splitting, sij denotes the invariant of the emitting dipole
  double SubtractionTerms::IqgSplit(double yij, double yjk, int typeEv, int typeOr, double N, bool useMqq){

    double SL1=0,SL2=0,SL3=0,Cmqq=0,IqgS=0,C00,C01,C10;

    // GGG e30 antenna contants
    C00 = -0.500;
    C01 = 0.00;
    C10 = 1.00;
    // todo : where is nf?
    double NF = 5;
 
    if( (yij < 0) || (yij > 1) ){
      printOut("VinciaSubtractionTerms::IqgSplit",
               "Input parameter non physical");
      return 0;
    }     
    if( typeOr==1 ){
      if( typeEv==1 ){
        // mqq strong for pT strong
        if( 4*yij < 1 ){
          SL1 = SubtractionTerms::SL1Stmqq( 4*yij  );
          SL2 = SubtractionTerms::SL2Stmqq( 4*yij  );
          SL3 = SubtractionTerms::SL3Stmqq( 4*yij  );
          Cmqq = SubtractionTerms::CStmqq( 4*yij,C00,C01,C10  );
          IqgS = 2*yij/(yij+yjk)*( 1/4.*SL1 - SL2 + SL3 + Cmqq );
        }
        if( 4*yjk < 1 ){
          SL1 = SubtractionTerms::SL1Stmqq( 4*yjk  );
          SL2 = SubtractionTerms::SL2Stmqq( 4*yjk  );
          SL3 = SubtractionTerms::SL3Stmqq( 4*yjk  );
          Cmqq = SubtractionTerms::CStmqq( 4*yjk,C00,C01,C10  );
          IqgS = IqgS + 2*yjk/(yij+yjk)*( 1/4.*SL1 - SL2 + SL3 + Cmqq );          
        }
      }   
      else if( typeEv==2 ){
        // mqq strong for mD strong
        if( yij == yjk ){ printOut("VinciaSubtractionTerms::IqgSplit",
                        "Invariants are equal, cannot determine min.");
          return 0;
	}
        else if( max( yij, yjk )/min( yij, yjk) > 1 ){  
          SL1 = SubtractionTerms::SL1Stmqq( min(yij,yjk)/max(yij,yjk)  );
          SL2 = SubtractionTerms::SL2Stmqq( min(yij,yjk)/max(yij,yjk)  );
          SL3 = SubtractionTerms::SL3Stmqq( min(yij,yjk)/max(yij,yjk)  );
          Cmqq = SubtractionTerms::CStmqq( min(yij,yjk)/max(yij,yjk),C00,C01,C10 );
          IqgS = 2*min(yij,yjk)/(yij+yjk)*( 1/4.*SL1 - SL2 + SL3 + Cmqq );
	}
      }   
      else { 
        printOut("VinciaSubtractionTerms::IqgSplit",
                      "Unknown evolution type."); 
        return 0;
      } 
    }
    else if ( typeOr==2 && useMqq ) {
      if ( typeEv==1 ) {
        // mqq smooth pT
        SL1 = SubtractionTerms::SL1Smmqq( yij,N );
        SL2 = SubtractionTerms::SL2Smmqq( yij,N );
        SL3 = SubtractionTerms::SL3Smmqq( yij,N );
        Cmqq = SubtractionTerms::CSmmqq( yij,N,C00,C01,C10 );
        IqgS = 2*yij/(yij+yjk)*(1/4.*SL1 - SL2 + SL3 + Cmqq );
        SL1 = SubtractionTerms::SL1Smmqq( yjk,N );
        SL2 = SubtractionTerms::SL2Smmqq( yjk,N );
        SL3 = SubtractionTerms::SL3Smmqq( yjk,N );
        Cmqq = SubtractionTerms::CSmmqq( yjk,N,C00,C01,C10 );
        IqgS = IqgS + 2*yjk/(yij+yjk)*(1/4.*SL1 - SL2 + SL3 + Cmqq );
      }
      else if ( typeEv==2 ) {
        // mqq smooth mD 
        if( yij == yjk ){ printOut("VinciaSubtractionTerms::IqgSplit",
                     "Invariants are equal, cannot determine min.");
          return 0;
	}
        else{
          SL1 = SubtractionTerms::SL1Smmqq( min( yij, yjk)/yij,N );
          SL2 = SubtractionTerms::SL2Smmqq( min( yij, yjk)/yij,N );
          SL3 = SubtractionTerms::SL3Smmqq( min( yij, yjk)/yij,N );
          Cmqq = SubtractionTerms::CSmmqq( min( yij, yjk)/yij,N,C00,C01,C10 );
          IqgS = 2*yjk/(yij+yjk)*(1/4.*SL1 - SL2 + SL3 + Cmqq );
          SL1 = SubtractionTerms::SL1Smmqq( min( yij, yjk)/yjk,N );
          SL2 = SubtractionTerms::SL2Smmqq( min( yij, yjk)/yjk,N );
          SL3 = SubtractionTerms::SL3Smmqq( min( yij, yjk)/yjk,N );
          Cmqq = SubtractionTerms::CSmmqq( min( yij, yjk)/yjk,N,C00,C01,C10 );
          IqgS = IqgS + 2*yij/(yij+yjk)*(1/4.*SL1 - SL2 + SL3 + Cmqq );
        }        
      }
      else { 
        printOut("VinciaSubtractionTerms::IqgSplit",
                      "Unknown evolution type."); 
        return 0;
      } 
    }
    else if (typeOr == 2 && !useMqq) {
      if( typeEv == 1 ){
        // pT smooth - gluon split in pT  
        SL1 = SubtractionTerms::SL1SmpT(yij);
        SL2 = SubtractionTerms::SL2SmpT(yij);
        SL3 = SubtractionTerms::SL3SmpT(yij);
        Cmqq = SubtractionTerms::CSmpT( yij,C00,C01,C10 ); 	      
	IqgS = 2*yij/(yij+yjk)*( 1./8.*SL1 - 1./2.*SL2 + SL3 + Cmqq);
        SL1 = SubtractionTerms::SL1SmpT(yjk);
        SL2 = SubtractionTerms::SL2SmpT(yjk);
        SL3 = SubtractionTerms::SL3SmpT(yjk);
        Cmqq = SubtractionTerms::CSmpT( yjk,C00,C01,C10 ); 
	IqgS += 2*yjk/(yij+yjk)*( 1./8.*SL1 - 1./2.*SL2 + SL3 + Cmqq);	      
      } 
      else { 
        printOut("VinciaSubtractionTerms::IqgSplit",
                      "Unknown evolution type."); 
        return 0;
      } 
    }   
    else { 
      printOut("VinciaSubtractionTerms::IqgSplit",
               "Unknown evolution type."); 
      return 0;
    } 

    IqgS = NF*IqgS;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::IqgSplit",
                              "IqgS = "+num2str(IqgS));    
    }
    return IqgS;
  }
  
  //=====================================================================
  // Base integrals to construct expanded Sudakovs

  //========
  // First set :: pT strong
  // Calculated with yE = 4 yij yjk
  // Rescale with yE -> 4 yE/N for yE = N yij yjk
 
  // Evolution variable
  double SubtractionTerms::yEpT(double yij, double yjk){ 

    if( (yij < 0) || (yjk < 0) || (yij+yjk > 1) ){
      printOut("VinciaSubtractionTerms::yE",
               "Input parameter(s) non physical");
      return 0;
    }   
      if( verbose == 9 ){   printOut("VinciaSubtractionTerms::yEpT",
                                     "yE = "+num2str(4*yij*yjk));
      }
    return 4*yij*yjk;
  }

  // Double log term for pT strong
  double SubtractionTerms::DLStpT(double y3){

    double DL;  
    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::DLStpT", 
               "Input parameter non physical");
      return 0;
    }     
    DL = -(log(4./y3)*log(-((-2. + 2.*sqrt(1 - y3) + y3)/y3)))/2. + 
      Li2((1 - sqrt(1 - y3))/2.) - Li2((1. + sqrt(1. - y3))/2.);

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::DLStpT",
                                   "DL = "+num2str(DL));
    }
    return DL;
  }
  
  // First subleading term for pT strong
  //  double SubtractionTerms::SL1StpT(double y3){

  //double SL1;
  //if( y3 < 0 || y3 > 1){
  //  printOut("VinciaSubtractionTerms::SL1StpT", 
  //           "Input parameter non physical");
  //  return 0;
  //}     
  //SL1 = 2.*(-2*sqrt(1. - y3) + 2*log(1. + sqrt(1. - y3)) - log(y3));
    
  //  if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL1StpT",
  //                               "SL1 = "+num2str(SL1));
  //}
  //return SL1;
  //}

  // Second subleading term for pT strong
  //double SubtractionTerms::SL2StpT(double y3){ 

  //double SL2;
  //if( y3 < 0 || y3 > 1){
  //  printOut("VinciaSubtractionTerms::SL2StpT", 
  //           "Input parameter non physical");
  //  return 0;
  //}     
  //SL2 = 2.*(-sqrt(1. - y3)/2. + log(1. + sqrt(1. - y3))/2. - log(y3)/4.);

  //if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL2StpT",
  //                               "SL2 = "+num2str(SL2));
  //}
  //return SL2;
  //}

  // First and second subleading term for pT strong
  // Added version of ( -SL1 + SL2 ) which improves numerical stability
  double SubtractionTerms::SL12StpT(double y3){

    // SL12 = -SL1 + SL2
    double SL12;
    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::SL12StpT", 
               "Input parameter non physical");
      return 0;
    }     
    SL12 = (3*(2*sqrt(1 - y3) - 2*log(1 + sqrt(1 - y3)) + log(y3)))/2.;
      
    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL12StpT",
                                   "SL12 = "+num2str(SL12));
    }
    return SL12;
  }

  // Third subleading term for pT strong
  double SubtractionTerms::SL3StpT(double y3){

    double SL3;

    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::SL3StpT",
               "Input parameter non physical");
      return 0;
    }     
    SL3 = (-13*sqrt(1 - y3))/36. + (sqrt(1 - y3)*y3)/36. + 
                            log(1 + sqrt(1 - y3))/3. - log(y3)/6.;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL3StpT",
                                   "SL3 = "+num2str(SL3));
    }
    return SL3;
  }

  // Integrated finite terms for pT strong
  double SubtractionTerms::CStpT(double y3, double C00, double C01, double C10){

    double CpT;
    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::CStpT",
               "Input parameter non physical");
      return 0;
    }     
    CpT = (2*(3*C00 - (C01 + C10)*(-1 + y3))*sqrt(1 - y3) - 
               3*C00*y3*log((2 + 2*sqrt(1 - y3) - y3)/y3))/24.;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::CStpT",
                              "CpT = "+num2str(CpT));    
    }
    return CpT;
  }

  //========
  // Second set :: dipole mass
  // Calculated for yE = 2 min( yij, yjk) 
  // Rescale yE -> 2 yE/N  for yE = N min( yij, yjk) 

  // Double log term for mD 
  double SubtractionTerms::DLStmD(double xi){

    double DL;
    if( xi < 0 || xi > 1){
      printOut("VinciaSubtractionTerms::DLStmD", 
               "Input parameter non physical");
      return 0;
    }     
    DL = -(log(1 + 1/xi)*log(xi)) - Li2(1/(1 + xi)) + Li2(xi/(1 + xi));

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::DLStmD",
                           "DL = "+num2str(DL));
    }  
    return DL;
  }

  // Subleading term 1 for strong mD 
  double SubtractionTerms::SL1StmD(double xi){

    double SL1;
    if( xi < 0 || xi > 1){
      printOut("VinciaSubtractionTerms::SL1StmD", 
               "Input parameter non physical");
      return 0;
    }     
    SL1 = (2*(-1 + xi - log(xi)))/(1 + xi);

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL1StmD",
                             "SL1 = "+num2str(SL1));
    }
    return SL1;
  }

  // Subleading term 2 for strong mD 
  double SubtractionTerms::SL2StmD(double xi){

    double SL2;
    if( xi < 0 || xi > 1){
      printOut("VinciaSubtractionTerms::SL2StmD", 
               "Input parameter non physical");
      return 0;
    }     
    SL2 = (3*(-1 + pow2(xi)) - 2*(1 + 2*xi)*log(xi))/(4.*pow2(1 + xi));

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL2StmD",
                             "SL2 = "+num2str(SL2));
    }
    return SL2;
  }

  // Subleading term 3 for strong mD 
  double SubtractionTerms::SL3StmD(double xi){

    double SL3;
    if( xi < 0 || xi > 1){
      printOut("VinciaSubtractionTerms::SL3StmD", 
               "Input parameter non physical");
      return 0;
    }     
    SL3 = ((-1 + xi)*(11 + xi*(20 + 11*xi)) - 6*(1 + 3*xi*(1 + xi))*log(xi))/
              	(36.*pow3(1 + xi));

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL3StmD",
                            "SL3 = "+num2str(SL3));
    }
    return SL3;
  }

  // Integrated finite terms for strong mD 
  double SubtractionTerms::CStmD(double xi, double C00, double C01, double C10){

    double Cm;
    if( xi < 0 || xi > 1){
      printOut("VinciaSubtractionTerms::SL3StmD", "Input parameter non physical");
      return 0;
    }     
    Cm = C10/(12.*pow3(1 + xi)) - (C10*pow2(xi))/(4.*pow3(1 + xi)) + 
           (C10*pow3(xi))/(6.*pow3(1 + xi)) + C00/(4.*pow2(1 + xi)) - 
           (C00*xi)/(2.*pow2(1 + xi)) + 
           (C00*pow2(xi))/(4.*pow2(1 + xi)) + 
	(C01*pow2(-1 + xi)*(1 + 2*xi))/(12.*pow3(1 + xi)); 

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::CStmD",
                             "Cm = "+num2str(Cm));
    }
    return Cm;
  }

  //========
  // Third set :: energy ordering 
  // Calculated for yE = ( yij + yjk)^2 

  // Cutoff on Energy in pT
  const double SubtractionTerms::del = 0.0001;

  // Evolution Variable for En
  double SubtractionTerms::yEEn( double yij, double yjk){
    if( ( yij < 0 ) || (yjk < 0 ) || (yij+yjk > 1 ) ){
      printOut("VinciaSubtractionTerms::yEEn",
               "Input parameter non physical");
      return 0;
    }
    
    if( verbose == 9 ){ printOut("VinciaSubtractionTerms::yEEn",
			      "yEEn = "+num2str(yij+yjk));
    }
    return yij+yjk;
  }

  // Double log term for En 
  double SubtractionTerms::DLEn(double y3){

    double DL;
    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::DLEn", 
               "Input parameter non physical");
      return 0;
    }     
    DL = 4*log(2)*log(1 + sqrt(1 - del)) - 2*log(1 + sqrt(1 - del))*log(del) + 
           2*log(del)*log(y3) - 4*log(2)*log(y3 + sqrt(-del + pow2(y3))) + 
           2*log(del)*log(y3 + sqrt(-del + pow2(y3))) - 
           4*log(y3)*log(y3 + sqrt(-del + pow2(y3))) - 
           Li2(-(pow2(1 - sqrt(1 - del))/del)) + 
           Li2(-(pow2(1 + sqrt(1 - del))/del)) + 
           Li2(-(pow2(y3 - sqrt(-del + pow2(y3)))/del)) - 
	Li2(-(pow2(y3 + sqrt(-del + pow2(y3)))/del)); 

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::DLEn",
                           "DL = "+num2str(DL));
    }
    return DL;
  }

  // Subleading term 1 for energy 
  double SubtractionTerms::SL1En(double y3){

    double SL1;

    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::SL1En", 
               "Input parameter non physical");
      return 0;
    }     
    SL1 = 2*(-2*sqrt(1 - del) + 2*sqrt(-del + pow2(y3)) + 
            log((2 + 2*sqrt(1 - del) - del)/del) - 
	       y3*log((-del + 2*y3*(y3 + sqrt(-del + pow2(y3))))/del)); 

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL1En","SL1 = "
      				  +num2str(SL1));
    }
    return SL1;
  }

  // Subleading term 2 for energy 
  double SubtractionTerms::SL2En(double y3){

    double SL2;

    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::SL2En", 
               "Input parameter non physical");
      return 0;
    }     
    SL2 = -sqrt(1 - del) + y3*sqrt(-del + pow2(y3)) - log(1 - sqrt(1 - del))/4. + 
            log(1 + sqrt(1 - del))/4. + log(2 + 2*sqrt(1 - del) - del)/4. - 
            log(del)/4. + (pow2(y3)*log(del))/4. + 
            (pow2(y3)*log(1 - sqrt(-del + pow2(y3))/y3))/4. - 
            (pow2(y3)*log(1 + sqrt(-del + pow2(y3))/y3))/4. - 
	    (pow2(y3)*log(-del + 2*y3*(y3 + sqrt(-del + pow2(y3)))))/4.; 

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL2En",
                             "SL2 = "+num2str(SL2));
    }
    return SL2;
  }

  //========
  // Fourth set :: pT smooth
  // Calculated with PimppT = y3ij y3jk / ( y3ij y3jk + y1 y2/y3ij^2 )
  // for branching y3ij

  // Double log term for pT smooth
  double SubtractionTerms::DLSmpT(double y3){

    double DL;  
    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::DLSmpT", 
               "Input parameter non physical");
      return 0;
    }     
    DL = -Li2(-2/(-1 + sqrt(1 + 4*y3))) - Li2(2/(1 + sqrt(1 + 4*y3)));

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::DLSmpT",
                           "DL = "+num2str(DL));
    }
    return DL;
  }
  
  // First subleading term for pT smooth
  double SubtractionTerms::SL1SmpT(double y3){

    double SL1;
    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::SL1SmpT", 
               "Input parameter non physical");
      return 0;
    }     
    SL1 = -4 - 2.*sqrt(1 + 4*y3)*log((-1 + sqrt(1 + 4*y3))/(1 + sqrt(1 + 4*y3)));

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL1SmpT",
                              "SL1 = "+num2str(SL1));
    }
    return SL1;
  }

  // Second subleading term for pT smooth
  double SubtractionTerms::SL2SmpT(double y3){ 

    double SL2;
    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::SL2SmpT", 
               "Input parameter non physical");
      return 0;
    }     
    SL2 = -1 - 0.5*sqrt(1 + 4*y3)*log((-1 + sqrt(1 + 4*y3))/(1 + sqrt(1 + 4*y3)));

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL2SmpT",
                            "SL2 = "+num2str(SL2));
    }
    return SL2;
  }

  // Third subleading term for pT smooth
  double SubtractionTerms::SL3SmpT(double y3){

    double SL3;

    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::SL3SmpT", 
               "Input parameter non physical");
      return 0;
    }     
    SL3 = (-(sqrt(1 + 4*y3)*(13 + 12*y3)) + 
            3*(1 + y3)*(1 + 4*y3)*(-log(1 - sqrt(1 + 4*y3)/(1 + 2*y3)) + 
	    log(1 + sqrt(1 + 4*y3)/(1 + 2*y3))))/(36.*sqrt(1 + 4*y3));

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL3SmpT",
                          "SL3 = "+num2str(SL3));
    }
    return SL3;
  }

  // Integrated finite terms for pT smooth
  double SubtractionTerms::CSmpT(double y3, double C00, double C01, double C10){

    double C;
    if( y3 < 0 || y3 > 1){
      printOut("VinciaSubtractionTerms::CSmpT", 
               "Input parameter non physical");
      return 0;
    }     
    C = (3.*C00 + C01 + C10 + 12.*(C01 + C10)*y3 - 
          6.*(C01 + C10)*y3*sqrt(1 + 4.*y3)*
          (-log(1. - 1./sqrt(1. + 4.*y3)) + log(1. + 1./sqrt(1. + 4.*y3))) + 
          6.*C00*y3*(Li2(-2./(-1. + sqrt(1. + 4.*y3))) + 
	       Li2(2./(1. + sqrt(1. + 4.*y3)))))/12.;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::CSmpT",
                             "C = "+num2str(C));    
    }
    return C;
  }

  //========
  // Fifth set :: smooth mD
  // Calculated for Pimp = min(m2)/( min(m2) + min(s1,s2)  )

  // Double log term for smooth mD 
  double SubtractionTerms::DLSmmD(double xi){

    double DL;
    if( xi < 0 || xi > 1){
      printOut("VinciaSubtractionTerms::DLSmmD", 
               "Input parameter non physical");
      return 0;
    }     
    DL = log(2)*log(1 + 1/xi) - Li2(-1/(2.*xi)) - Li2(1/(1 + xi)) + 
	   Li2(1/(2 + 2*xi));

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::DLSmmD",
                           "DL = "+num2str(DL));
    }
    return DL;
  }

  // Subleading term 1 for smooth mD 
  double SubtractionTerms::SL1SmmD(double xi){

    double SL1;
    if( xi < 0 || xi > 1){
      printOut("VinciaSubtractionTerms::SL1SmmD", 
               "Input parameter non physical");
      return 0;
    }     
    SL1 = -1 + log(2 + 1/xi) + (xi*log(4)*log(1 - 1/(1 + 2*xi)))/2. - 
           (xi*log(4)*log(1 + 1/(1 + 2*xi)))/2. - 2*xi*log(1 - 1/(1 + 4*xi)) + 
           2*xi*log(1 + 1/(1 + 4*xi)) + xi*Li2(-1/(2.*xi)) 
	+ xi*Li2(1/(1 + xi)) - xi*Li2(1/(2 + 2*xi));

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL1SmmD",
                           "SL1 = "+num2str(SL1));
    }
    return SL1;
  }

  // Subleading term 2 for smooth mD 
  double SubtractionTerms::SL2SmmD(double xi){

    double SL2;
    if( xi < 0 || xi > 1){
      printOut("VinciaSubtractionTerms::SL2SmmD", 
               "Input parameter non physical");
      return 0;
    }     
    SL2 =( -0.75 - xi*log(2) + log(2 + 1/xi)/2. 
            - (pow2(xi)*log(4)*log(1- 1/(1 + 2*xi)))/2. 
            + (pow2(xi)*log(4)*log(1 + 1/(1 + 2*xi)))/2. 
            - xi*log(1 - 1/(1 + 4*xi)) + xi*log(1 + 1/(1 + 4*xi)) 
            - pow2(xi)*Li2(-1/(2.*xi)) - pow2(xi)*Li2(1/(1 + xi)) 
	   + pow2(xi)*Li2(1/(2 + 2*xi)))/2;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL2SmmD","SL2 = "
      				  +num2str(SL2));
    }
    return SL2;
  }

  // Subleading term 3 for smooth mD 
  double SubtractionTerms::SL3SmmD(double xi){

    double SL3;
    if( xi < 0 || xi > 1){
      printOut("VinciaSubtractionTerms::SL3SmmD", 
               "Input parameter non physical");
      return 0;
    }     
    SL3 = -0.10416666666666667 + xi/8. + log(2)/6. - (xi*log(2))/4. 
            + (pow2(xi)*log(2))/2. + (pow3(xi)*log(4)*log(1 - 1/(1 + 2*xi)))/4. 
            - (pow3(xi)*log(4)*log(1 + 1/(1 + 2*xi)))/4. 
            + (-29 - 6*xi*(5 + 4*xi) 
            + 24*(1 + 2*xi)*(1 + xi + pow2(xi))*(-log(1 - 1/(1 + 4*xi)) 
            + log(1 + 1/(1 + 4*xi))))/144. + (pow3(xi)*Li2(-1/(2.*xi)))/2.
            + (pow3(xi)*Li2(1/(1 + xi)))/2.- (pow3(xi)*Li2(1/(2 + 2*xi)))/2.;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL3SmmD",
                             "SL3 = "+num2str(SL3));
    }
    return SL3;
  }

  // Integrated finite terms for smooth mD 
  double SubtractionTerms::CSmmD(double xi, double C00, double C01, double C10){

    double Cm;
    if( xi < 0 || xi > 1){
      printOut("VinciaSubtractionTerms::SL3SmmD", "Input parameter non physical");
      return 0;
    }     
    Cm = (C10 + 6*C00*(1 + 4*xi) + 3*(1 + 4*xi)*(C01 - 2*C10*xi) - 
            12*xi*(1 + 2*xi)*(2*C00 + C01 - 2*C10*xi)*
            (-log(1 - 1/(1 + 4*xi)) + log(1 + 1/(1 + 4*xi))))/48.;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::CSmmD","Cm = "
      				  +num2str(Cm));
    }
    return Cm;
  }

  //========
  // Sixth set : strong mqq

  // Subleading term 1 for strong mqq 
  double SubtractionTerms::SL1Stmqq(double y3){

    double SL1;
    if( y3 < 0 || y3 > 1 ){
      printOut("VinciaSubtractionTerms::SL1Stmqq", "Input parameter non physical");
      return 0;
    }     
    SL1 = -1 + y3 - log(y3);

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL1Stmqq","SL1 = "
      				  +num2str(SL1));
    }
    return SL1;
  }

  // Subleading term 2 for strong mqq 
  double SubtractionTerms::SL2Stmqq(double y3){

    double SL2;
    if( y3 < 0 || y3 > 1 ){
      printOut("VinciaSubtractionTerms::SL2Stmqq", "Input parameter non physical");
      return 0;
    }     
    SL2 = (-3 + 4*y3 - pow2(y3) - 2*log(y3))/8.;

   if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL2Stmqq","SL2 = "
      				  +num2str(SL2));
    }
    return SL2;
  }

  // Subleading term 3 for strong mqq 
  double SubtractionTerms::SL3Stmqq(double y3){

    double SL3;
    if( y3 < 0 || y3 > 1 ){
      printOut("VinciaSubtractionTerms::SL3Stmqq", "Input parameter non physical");
      return 0;
    }     
    SL3 = (-11 + 18*y3 - 9*pow2(y3) + 2*pow3(y3) - 6*log(y3))/36.;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL3Stmqq","SL3 = "
      				  +num2str(SL3));
    }
    return SL3;
  }

  // Integrated finite terms for strong mqq 
  double SubtractionTerms::CStmqq(double y3, double C00, double C01, double C10){

    double Cm;
    if( y3 < 0 || y3 > 1 ){
      printOut("VinciaSubtractionTerms::SL3Stmqq", "Input parameter non physical");
      return 0;
    }     
    Cm =  (pow2(-1 + y3)*(3*C00 + C01 + C10 + 2*C01*y3 - C10*y3))/12.;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::CStmqq","Cm = "
      				  +num2str(Cm));
    }
    return Cm;
  }

  //========
  // Seventh set : mqq for smooth pT
  // Pimp = N yij yjk/ ( N yij yjk + mqq )

  // N in evolution variable for pT ordering
  //const double SubtractionTerms::NpT = 4;

  // Subleading term 1 for smooth pT for mqq 
  double SubtractionTerms::SL1SmpTmqq(double y3, double NpT){

    double SL1;
    if( y3 < 0 || y3 > 1 ){
      printOut("VinciaSubtractionTerms::SL1SmpTmqq", "Input parameter non physical");
      return 0;
    }     
    SL1 = -1 + log(1 + 1/(NpT*y3)) - NpT*y3*log(1 - 1/(1 + 2*NpT*y3)) + 
	      NpT*y3*log(1 + 1/(1 + 2*NpT*y3));

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL1SmpTmqq","SL1 = "
      				  +num2str(SL1));
    }
    return SL1;
  }

  // Subleading term 2 for smooth pT for mqq 
  double SubtractionTerms::SL2SmpTmqq(double y3, double NpT){

    double SL2;
    if( y3 < 0 || y3 > 1 ){
      printOut("VinciaSubtractionTerms::SL2SmpTmqq", "Input parameter non physical");
      return 0;
    }     
    SL2 = (-3 - 2*NpT*y3 + 2*pow2(1 + NpT*y3)*
             log((1 + 1/(1 + 2*NpT*y3))/(1 - 1/(1 + 2*NpT*y3))))/8.;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL2SmpTmqq","SL2 = "
      				  +num2str(SL2));
    }
    return SL2;
  }

  // Subleading term 3 for smooth pT for mqq 
  double SubtractionTerms::SL3SmpTmqq(double y3, double NpT){

    double SL3;
    if( y3 < 0 || y3 > 1 ){
      printOut("VinciaSubtractionTerms::SL3SmpTmqq", "Input parameter non physical");
      return 0;
    }     
    SL3 = (-11 - 3*NpT*y3*(5 + 2*NpT*y3) + 
	     6*pow3(1 + NpT*y3)*log((1 + 1/(1 + 2*NpT*y3))/
                        (1 - 1/(1 + 2*NpT*y3))))/36.;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL3SmpTmqq","SL3 = "
      				  +num2str(SL3));
    }
    return SL3;
  }

  // Integrated finite terms for strong mqq 
  double SubtractionTerms::CSmpTmqq(double y3, double NpT, double C00, double C01, double C10){

    double Cm;
    if( y3 < 0 || y3 > 1 ){
      printOut("VinciaSubtractionTerms::SL3SmpTmqq", "Input parameter non physical");
      return 0;
    }     
    Cm =  (C00*(1 + 2*NpT*y3 - 2*NpT*y3*(1 + NpT*y3)*
            log((1 + 1/(1 + 2*NpT*y3))/(1 - 1/(1 + 2*NpT*y3)))))/4. + 
            (C01*(1 - 3*NpT*y3 - 6*pow2(NpT)*pow2(y3) + 
            6*pow2(NpT)*pow2(y3)*(1 + NpT*y3)*
            log((1 + 1/(1 + 2*NpT*y3))/(1 - 1/(1 + 2*NpT*y3)))))/12. + 
            (C10*(2 + 3*NpT*y3*(3 + 2*NpT*y3) - 
            6*NpT*y3*pow2(1 + NpT*y3)*
		  log((1 + 1/(1 + 2*NpT*y3))/(1 - 1/(1 + 2*NpT*y3)))))/24. ;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::CSmpTmqq","Cm = "
      				  +num2str(Cm));
    }
  return Cm;
  }

  //========
  // Eighth set : mqq for smooth mD
  // Pimp = N min(y3ij,y3jk)/ ( N min(y3ij,y3jk) + mqq )

  // N in evolution variable for smooth mD ordering
  //const double SubtractionTerms::NmD = 2;

  // Subleading term 1 for smooth mD for mqq 
  double SubtractionTerms::SL1Smmqq(double xi, double NmD){

    double SL1;
    if( xi < 0 || xi > 1 ){
      printOut("VinciaSubtractionTerms::SL1SmmDmqq", "Input parameter non physical");
      return 0;
    }     
    SL1 = -1 + log(1 + 1/(NmD*xi)) - NmD*xi*log(1 - 1/(1 + 2*NmD*xi))+ 
      NmD*xi*log(1 + 1/(1 + 2*NmD*xi));

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL1SmmDmqq","SL1 = "
      				  +num2str(SL1));
    }
    return SL1;
  }

  // Subleading term 2 for smooth mD for mqq 
  double SubtractionTerms::SL2Smmqq(double xi, double NmD){

    double SL2;
    if( xi < 0 || xi > 1 ){
      printOut("VinciaSubtractionTerms::SL2SmmDmqq", "Input parameter non physical");
      return 0;
    }     
    SL2 =(-3 - 2*NmD*xi + 2*pow2(1 + NmD*xi)*
	  (-log(1 - 1/(1 + 2*NmD*xi)) + log(1 + 1/(1 + 2*NmD*xi))))/8.;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL2SmmDmqq","SL2 = "
      				  +num2str(SL2));
    }
    return SL2;
  }

  // Subleading term 3 for smooth mD for mqq 
  double SubtractionTerms::SL3Smmqq(double xi, double NmD){

    double SL3;
    if( xi < 0 || xi > 1 ){
      printOut("VinciaSubtractionTerms::SL3SmmDmqq", "Input parameter non physical");
      return 0;
    }     
    SL3 = (-11 - 3*NmD*xi*(5 + 2*NmD*xi) + 6*pow3(1 + NmD*xi)*
	   (-log(1 - 1/(1 + 2*NmD*xi)) + log(1 + 1/(1 + 2*NmD*xi))))/36.;

    if( verbose == 9 ){   printOut("VinciaSubtractionTerms::SL3SmmDmqq","SL3 = "
      				  +num2str(SL3));
    }
    return SL3;
  }


  // Integrated finite terms for smooth mD mqq 
  double SubtractionTerms::CSmmqq(double xi, double NmD, double C00, double C01, double C10){

    double Cm;
    if( xi < 0 || xi > 1 ){
      printOut("VinciaSubtractionTerms::SL3SmmDmqq", "Input parameter non physical");
      return 0;
    }     
   Cm = (C00*(1 + 2*NmD*xi - 2*NmD*xi*(1 + NmD*xi)*
         (-log(1 - 1/(1 + 2*NmD*xi)) + log(1 + 1/(1 + 2*NmD*xi)))))/4. + 
        (C01*(1 - 3*NmD*xi - 6*pow2(NmD)*pow2(xi) + 
        6*pow2(NmD)*pow2(xi)*(1 + NmD*xi)*
         (-log(1 - 1/(1 + 2*NmD*xi)) + log(1 + 1/(1 + 2*NmD*xi)))))/12. + 
        (C10*(2 + 3*NmD*xi*(3 + 2*NmD*xi) - 
        6*NmD*xi*pow2(1 + NmD*xi)*
	      (-log(1 - 1/(1 + 2*NmD*xi)) + log(1 + 1/(1 + 2*NmD*xi)))))/24.;

    if( verbose == 9 ){ printOut("VinciaSubtractionTerms::CSmmDmqq","Cm = "+num2str(Cm));
    }
  return Cm;
}



}



